#Imports
import matplotlib.pyplot as plt
import matplotlib.patches as patches
from PIL import Image
import numpy as np

def plot_boxes(im, pred_boxes , pred_class , output): # Write the required arguments
  # The function should plot the predicted boxes on the images and save them.
  # Tip: keep the dimensions of the output image less than 800 to avoid RAM crashes.
  im = np.transpose(im , (1,2,0))
  print(im.shape)
  fig,ax = plt.subplots(1)
  ax.imshow(im)
  count = 0
  for box in pred_boxes:
    count = count + 1
    if count > 5:
      break
    rect = patches.Rectangle(box[0] , box[1][0]-box[0][0] , box[1][1]-box[0][1], linewidth=1 , edgecolor='r',facecolor='none')
    ax.add_patch(rect)
    plt.text(box[0][0] , box[0][1] , pred_class[count-1] ,color='r')

  plt.savefig(output)
  return




#Imports
from PIL import Image
import numpy as np
import jsonlines
import json


class Dataset(object):
    '''
        A class for the dataset that will return data items as per the given index
    '''

    def __init__(self, annotation_file, transforms = None):
        '''
            Arguments:
            annotation_file: path to the annotation file
            transforms: list of transforms (class instances)
                        For instance, [<class 'RandomCrop'>, <class 'Rotate'>]
        '''
        self.annotation_file = annotation_file
        self.transforms = transforms

        

    def __len__(self):
        '''
            return the number of data points in the dataset
        '''
        with open(self.annotation_file) as f:
            annotation_list = f.readlines()
            length = len(annotation_list)
        return length
        

    def __getitem__(self, idx):
        '''
            return the dataset element for the index: "idx"
            Arguments:
                idx: index of the data element.

            Returns: A dictionary with:
                image: image (in the form of a numpy array) (shape: (3, H, W))
                gt_bboxes: N X 5 array where N is the number of bounding boxes, each 
                            consisting of [class, x1, y1, x2, y2]
                            x1 and x2 lie between 0 and width of the image,
                            y1 and y2 lie between 0 and height of the image.

            You need to do the following, 
            1. Extract the correct annotation using the idx provided.
            2. Read the image and convert it into a numpy array (wont be necessary
                with some libraries). The shape of the array would be (3, H, W).
            3. Scale the values in the array to be with [0, 1].
            4. Create a dictonary with both the image and annotations
            4. Perform the desired transformations.
            5. Return the transformed image and annotations as specified.
        '''
        images = []
        with jsonlines.open(self.annotation_file , 'r') as f:
            annotation_list = f.read()
            while idx > 0:
                annotation_list = f.read()
                idx = idx-1
            # reqd_data = annotation_list[idx]
            path = annotation_list["img_fn"]
            img = np.array(Image.open("./data/"+path))
            images.append(img)
            for transform in self.transforms:
                imag = transform(img)
                images.append(imag)
        return images , annotation_list


    


from .model import ObjectDetectionModel
from torch_package.data.transforms import BlurImage
from torch_package.data.transforms import CropImage
from torch_package.data.transforms import FlipImage
from torch_package.data.transforms import RescaleImage
from torch_package.data.transforms import RotateImage
from torch_package.data import Dataset
from torch_package.analysis import plot_boxes


#Imports
from PIL import Image, ImageFilter
from scipy.ndimage.filters import gaussian_filter
import numpy as np

class BlurImage(object):
    '''
        Applies Gaussian Blur on the image.
    '''

    def __init__(self, radius):
        '''
            Arguments:
            radius (int): radius to blur
        '''

        # Write your code here
        self.radius = radius
        

    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL Image)

            Returns:
            image (numpy array or PIL Image)
        '''

        # Write your code here
        blur_image = gaussian_filter(image , sigma = self.radius)
        return blur_image
        

# blur = BlurImage(2)

# img = np.array(Image.open('./data/imgs/0.jpg'))
# im = blur(img)

# image = Image.fromarray(im)
# image.save('blur.jpg')


#Imports
import numpy as np
from PIL import Image

class RescaleImage(object):
    '''
        Rescales the image to a given size.
    '''

    def __init__(self, output_size):
        '''
            Arguments:
            output_size (tuple (W,H)or int): Desired output size. If tuple, output is
            matched to output_size. If int, smaller of image edges is matched
            to output_size keeping aspect ratio the same.
        '''

        # Write your code here
        self.output_size = output_size

    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)

            Note: You do not need to resize the bounding boxes. ONLY RESIZE THE IMAGE.
        '''

        # Write your code here
        width, height = image.shape[1] , image.shape[0]
        image = Image.fromarray(image)
        if isinstance(self.output_size , int):
            if width < height:
                aspect_ratio = height/width
                width = self.output_size
                height = round(aspect_ratio * width)
                im = image.resize((width,height))
                im = np.array(im)
            else:
                aspect_ratio = width/height
                height = self.output_size
                width = round(aspect_ratio * height)
                im = image.resize((width,height))
                im = np.array(im)
            return im
        else:
            im = image.resize(self.output_size)
            im = np.array(im)
            return im


# rescale = RescaleImage(240)

# img = np.array(Image.open('./data/imgs/0.jpg'))
# im = rescale(img)
# print(im.shape)
# image = Image.fromarray(im)
# image.save('rescale.jpg')
 


#Imports
from PIL import Image
import numpy as np


class CropImage(object):
    '''
        Performs either random cropping or center cropping.
    '''

    def __init__(self, shape, crop_type='center'):
        '''
            Arguments:
            shape: output shape of the crop (h, w)
            crop_type: center crop or random crop. Default: center
        '''

        # Write your code here
        self.shape = shape
        self.crop_type = crop_type

    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''
        height, width = image.shape[0] , image.shape[1]
        new_width = self.shape[1]
        new_height = self.shape[0]
        if self.crop_type == 'center':
            left = round((width - new_width)/2)
            top = round((height - new_height)/2)
            x_right = round(width - new_width) - left
            x_bottom = round(height - new_height) - top
            right = width - x_right
            bottom = height - x_bottom
            im_crop = image[top:bottom , left:right]
            return im_crop
        else:
            im_crop = image[0: new_height , 0:new_width]
            return im_crop
        # Write your code here

        
# crop = CropImage([478,640] , 'random')

# img = np.array(Image.open('./data/imgs/0.jpg'))
# im = crop(img)

# image = Image.fromarray(im)
# image.save('crop.jpg')
 


#Imports
import numpy as np
from PIL import Image


class FlipImage(object):
    '''
        Flips the image.
    '''

    def __init__(self, flip_type='horizontal'):
        '''
            Arguments:
            flip_type: 'horizontal' or 'vertical' Default: 'horizontal'
        '''

        # Write your code here
        self.flip_type = flip_type

        
    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''

        # Write your code here
        if self.flip_type == 'vertical':
            im = np.flip(image , axis = 0)
            return im
        else:
            im = np.flip(image , axis = 1)
            return im


# flip = FlipImage('vertical')

# img = np.array(Image.open('./data/imgs/0.jpg'))
# im = flip(img)

# image = Image.fromarray(im)
# image.save('flip.jpg')


#Imports
from PIL import Image
import numpy as np

class RotateImage(object):
    '''
        Rotates the image about the centre of the image.
    '''

    def __init__(self, degrees):
        '''
            Arguments:
            degrees: rotation degree.
        '''
        
        # Write your code here
        self.degrees = degrees

    def __call__(self, sample):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''

        # Write your code here
        sample = Image.fromarray(sample)
        rotated_image = sample.rotate(self.degrees , expand = True)
        rotated_image = np.array(rotated_image)
        return rotated_image


# rotate = RotateImage(-90)

# img = np.array(Image.open('./data/imgs/0.jpg'))
# im = rotate(img)

# image = Image.fromarray(im)
# image.save('rotate.jpg')
 